package ru.dbotthepony.mc.otm.recipe

import com.mojang.serialization.Codec
import com.mojang.serialization.MapCodec
import net.minecraft.core.HolderLookup
import net.minecraft.core.RegistryAccess
import net.minecraft.core.component.DataComponents
import net.minecraft.network.FriendlyByteBuf
import net.minecraft.network.RegistryFriendlyByteBuf
import net.minecraft.network.codec.StreamCodec
import net.minecraft.world.item.ItemStack
import net.minecraft.world.item.crafting.CraftingBookCategory
import net.minecraft.world.item.crafting.CraftingInput
import net.minecraft.world.item.crafting.RecipeSerializer
import net.minecraft.world.item.crafting.ShapedRecipe
import net.minecraft.world.item.crafting.ShapedRecipePattern
import net.minecraft.world.item.enchantment.ItemEnchantments
import net.minecraft.world.level.Level
import ru.dbotthepony.mc.otm.capability.matteryEnergy
import ru.dbotthepony.mc.otm.container.util.iterator
import ru.dbotthepony.mc.otm.core.filterNotNull

class EnergyContainerRecipe(group: String, category: CraftingBookCategory, pattern: ShapedRecipePattern, result: ItemStack, showNotification: Boolean = true) : ShapedRecipe(group, category, pattern, result, showNotification) {
	constructor(parent: ShapedRecipe) : this(parent.group, parent.category(), parent.pattern, parent.result, parent.showNotification())

	override fun assemble(container: CraftingInput, registryAccess: HolderLookup.Provider): ItemStack {
		val itemStack = super.assemble(container, registryAccess)

		val battery = container.items().stream()
			.filter { !it.isEmpty }
			.map { it.matteryEnergy }
			.filterNotNull()
			.findAny().orElse(null)

		if (battery != null) {
			itemStack.matteryEnergy?.batteryLevel = battery.batteryLevel
		}

		if (itemStack.isEnchantable) {
			for (it in container.items()) {
				val enchantments = it[DataComponents.ENCHANTMENTS]

				if (enchantments != null) {
					itemStack[DataComponents.ENCHANTMENTS] = ItemEnchantments.Mutable(itemStack.getOrDefault(DataComponents.ENCHANTMENTS, ItemEnchantments.EMPTY)).let {
						for ((key, level) in enchantments.entrySet()) {
							it.upgrade(key, level)
						}

						it.toImmutable()
					}
				}
			}
		}

		return itemStack
	}

	override fun matches(container: CraftingInput, level: Level): Boolean {
		return super.matches(container, level) && container.items().none { it.isDamaged }
	}

	override fun getSerializer(): RecipeSerializer<EnergyContainerRecipe> {
		return Companion
	}

	companion object : RecipeSerializer<EnergyContainerRecipe> {
		private val codec by lazy {
			RecipeSerializer.SHAPED_RECIPE.codec().xmap(::EnergyContainerRecipe, { it })
		}

		private val streamCodec by lazy {
			RecipeSerializer.SHAPED_RECIPE.streamCodec().map(::EnergyContainerRecipe, { it })
		}

		override fun codec(): MapCodec<EnergyContainerRecipe> {
			return codec
		}

		override fun streamCodec(): StreamCodec<RegistryFriendlyByteBuf, EnergyContainerRecipe> {
			return streamCodec
		}
	}
}
